/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 8-10 - signalr5.c - Signal Handling in R4 and R5 apps
 *  
 *  HDR2: Chapter 8, page 214
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/*****  Include Required Header Files    *****************************/

#include <X11/Intrinsic.h>                /* The toolkit Intrinsics  */
#include <Xm/Xm.h>                        /* Motif symbols           */
#include <Xm/TextF.h>                     /* XmTextField             */
#include <signal.h>                       /* signals                 */

/*****  Declare Global Variables   ***********************************/

Widget toplevel, text;                    /* Widget tree             */
XtAppContext context;                     /* Application context     */
XtInputId inputid;                        /* Input fd/client_data ID */
int mypipe[2];                            /* Pipe to myself          */

/*****  Forward declare callback routines   **************************/

void SignalHandler();                     /* Signal handler          */
void SignalCallback();                    /* Input callback          */

/*****  Start of main()   ********************************************/

main( argc, argv )
int argc;
char ** argv;
{
/*****  Init. Toolkit, Appl. Context, Application Shell  *************/

  toplevel = XtAppInitialize(&context, "Signal", 
      NULL, 0, &argc, argv, NULL, NULL, 0);

  text = XtCreateManagedWidget("text", xmTextFieldWidgetClass,
      toplevel, NULL, 0);

  if (!pipe(mypipe  )){                 /* Create pipe, register end */
    signal(SIGINT, SignalHandler);      /* Register handler, callback*/
    inputid = XtAppAddInput(context, mypipe[0], XtInputReadMask, 
                     SignalCallback, NULL);
  }
  XtRealizeWidget(toplevel);
  XtAppMainLoop(context);
}

/* Callbacks and handlers   ******************************************/

void SignalHandler()                    /* Get signal, write to pipe */
{
  int signalid=SIGINT;
  write(mypipe[1], &signalid, sizeof(int));
}

void SignalCallback(client, fd, inputid)    /* Handle new pipe input */
XtPointer client;
int *fd;
XtInputId *inputid;
{
  char buffer[20];
  int signalid;
  read(*fd, &signalid, sizeof(int));
  sprintf(buffer, "Signal %i received", signalid);
  XtVaSetValues(text, XmNvalue, buffer, NULL);
}


